@@ -1,15 +1,15 @@
 package List::Maker;
 
-use version; $VERSION = qv('0.0.5');
+our $VERSION = '0.005';
 
 use warnings;
 use strict;
 use Carp;
+use List::Util qw<shuffle sum>;
 
 # Handle contextual returns
 sub _context {
     # Return original list in list context...
-
     return @_ if (caller 1)[5];
 
     # Otherwise, Anglicize list...
@@ -21,43 +21,67 @@ sub _context {
     return join($sep, @_[0..@_-2]) . $sep . "and $_[-1]";
 }
 
-# Regexes to parse the acceptable list syntaxes...
-my $NUM    = qr{\s* [+-]? \d+ (?:\.\d*)? \s* }xms;
-my $TO     = qr{\s* \.\. \s*}xms;
-my $FILTER = qr{ (?: : (.*) )? }xms;
+# General filters that can be applied...
+my %selector_sub = (
+    pick => sub {
+        return (shuffle @_)[0..shift()-1]
+    },
 
+    roll => sub {
+        return map { $_[rand @_] } 1..shift;
+    },
+
+    all => sub {
+        shift; return @_;
+    },
+);
+# Regexes to parse the acceptable list syntaxes...
+my $NUM      = qr{\s* [+-]? \d+ (?:\.\d*)? \s* }xms;
+my $TO       = qr{\s* \.\. \s*}xms;
+my $FILTER   = qr{ (?: : (?! \s* (?:pick|roll)) (.*?) )? }xms;
+my $SELECTOR = qr{ (?: : \s* (pick|roll) \s* (\d*) )? \s* }xms;
+
+# Mappings from specifications to handlers...
 my @handlers = (
     # <1, 2 .. 10>
-    { pat => qr{\A ($NUM) , ($NUM) ,? $TO (\^?) ($NUM) $FILTER \Z}xms,
-      gen => sub{ _gen_range(
-                    {from=>$1, to=>$4, by=>$2-$1, filter=>$5, exto=>$3}
-                  );
+    { pat => qr{\A ($NUM) , ($NUM) ,? $TO (\^?) ($NUM) $FILTER $SELECTOR \Z}xms,
+      gen => sub{ _gen_range({
+                    from=>$1, to=>$4, by=>$2-$1, exto=>$3,
+                    filter=>$5, selector => $6, count => $7,
+                  })
              },
     },
-    
-    # <1 .. 10 by 2> 
-    { pat => qr{\A ($NUM) (\^?) $TO (\^?) ($NUM) (?:(?:x|by) ($NUM))? $FILTER \Z}xms,
-      gen => sub{ _gen_range(
-                    {from=>$1, to=>$4, by=>$5, filter=>$6, exfrom=>$2, exto=>$3}
-                  );
+
+    # <1 .. 10 by 2>
+    { pat => qr{\A ($NUM) (\^?) $TO (\^?) ($NUM) (?:(?:x|by) ($NUM))? $FILTER $SELECTOR \Z}xms,
+      gen => sub{ _gen_range({
+                    from=>$1, to=>$4, by=>$5, exfrom=>$2, exto=>$3,
+                    filter=>$6, selector => $7, count => $8,
+                  });
              },
     },
-    
-    # <^7 by 2> 
-    { pat => qr{\A \s* \^ ($NUM) \s* (?:(?:x|by) \s* ($NUM))? $FILTER \Z}xms,
-      gen => sub{ _gen_range(
-                    {from=>0, to=>$1, by=>$2, filter=>$3, exto=>1}
-                  );
+
+    # <^7 by 2>
+    { pat => qr{\A \s* \^ ($NUM) \s* (?:(?:x|by) \s* ($NUM))? $FILTER $SELECTOR \Z}xms,
+      gen => sub{ _gen_range({
+                    from=>0, to=>$1, by=>$2, exto=>1,
+                    filter=>$3, selector => $4, count => $5,
+                  });
              },
     },
-    
-    # <^@foo> 
+
+    # <^@foo>
     { pat => qr{\A \s* \^ \s* ( (?:\S+\s+)* \S+) \s* \Z}xms,
-      gen => sub{ my @array = split /\s+/, $1;
-                  _gen_range( {from=>0, to=>@array-1});
+      gen => sub{
+                my $specification = $1;
+                $specification =~ s{$SELECTOR \Z}{}x;
+                my ($selector, $count) = ($selector_sub{$1||'all'}, $2||1);
+
+                my @array = split /\s+/, $specification;
+                $selector->($count, _gen_range( {from=>0, to=>@array-1}));
              },
     },
-    
+
     # MINrMAX random range notation
     { pat => qr/^\s* ([+-]?\d+(?:[.]\d*)?|[.]\d+) \s* r \s* ([+-]?\d+(?:[.]\d*)?|[.]\d+) \s* $/xms,
       gen => sub {
@@ -88,8 +112,6 @@ my @handlers = (
         }
 
         return @rolls if wantarray;
-
-        use List::Util qw( sum );
         return sum @rolls;
       }
     },
@@ -128,7 +150,7 @@ my $list_maker_sub = sub {
 
     # If it doesn't match a special form, it's a < word list >...
     for my $handler (@user_handlers, @handlers) {
-        next if $listspec !~ m{$handler->{pat}}xms;
+        next if $listspec !~ m{$handler->{pat} }xms;
         return $handler->{gen}();
     }
 
@@ -137,21 +159,32 @@ my $list_maker_sub = sub {
 
 sub import {
     shift; # Don't need package name
+    my $caller = caller;
 
-    # Explicit export requested
+    # Note lexical scope of import...
+    $^H{'List::Maker::is_active'} = 1;
+
+    # Explicit export(s) requested...
     if (@_) {
-        my $caller = caller;
         for my $name (@_) {
             no strict 'refs';
             *{$caller.'::'.$name} = $list_maker_sub;
         }
     }
+
+    # Otherwise use 'glob' (to provide magic behaviour as well)...
     else {
         my ($package, $file) = caller;
+
+        # Get as close to lexical behavior as per-5.10 will allow...
         $caller_expecting_special_behaviour{ $package, $file } = 1;
+
+        no strict 'refs';
+        *{$caller.'::glob'} = \&_glob_replacement;
     }
 }
 
+# Users can add their own handlers...
 sub add_handler {
     while (my ($regex, $sub) = splice @_, 0, 2) {
         croak "Usage: List::Make::add_handler(qr/.../, sub{...})\nError"
@@ -162,30 +195,48 @@ sub add_handler {
 }
 
 
+
+# This sub is used instead of globs the special behaviours...
 no warnings 'redefine';
-*CORE::GLOBAL::glob = sub
-{
+sub _glob_replacement {
     # Don't be magical in those files that haven't loaded the module...
-    my ($package, $file) = caller;
-    if (!$caller_expecting_special_behaviour{$package, $file}) {
-        use File::Glob;
-        goto &File::Glob::bsd_glob
+    my ($package, $file, $scope_ref) = (caller 0)[0,1,10];
+
+    # Check for lexical scoping (only works in 5.10 and later)...
+    my $in_scope = $] < 5.010 || $scope_ref && $scope_ref->{'List::Maker::is_active'};
+
+    # If not being magical...
+    if (!$caller_expecting_special_behaviour{$package, $file} || !$in_scope ) {
+        # Use any overloaded version of glob...
+        goto &CORE::GLOBAL::glob if exists &CORE::GLOBAL::glob;
+
+        # Otherwise, use the core glob behaviour...
+        use File::Glob 'csh_glob';
+        return &csh_glob;
     }
+
+    # Otherwise, be magical...
     else {
         goto &{$list_maker_sub};
     }
 };
 
+# Generate a range of values, selected or filtered as appropriate...
 sub _gen_range {
-    my ($from, $to, $incr, $filter, $exfrom, $exto)
-        = @{shift()}{ qw<from to by filter exfrom exto> };
+    my ($from, $to, $incr, $filter, $exfrom, $exto, $selector, $count)
+        = @{shift()}{ qw<from to by filter exfrom exto selector count> };
 
+    # Trim leading and trailing whitespace from endpoints...
     s/^ \s+ | \s+ $//gxms for $from, $to;
 
+    # Default increment is +/- 1, depending on end-points...
     if (!defined $incr) {
         $incr = -($from <=> $to);
     }
 
+    # Default count is 1...
+    $count ||= 1;
+
     # Check for nonsensical increments (zero or the wrong sign)...
     my $delta = $to - $from;
     croak sprintf "Sequence <%s, %s, %s...> will never reach %s",
@@ -195,20 +246,25 @@ sub _gen_range {
     # Generate unfiltered list of values...
     $from += $incr if $exfrom;
     my @vals;
+    # <N..N>
     if ($incr==0) {
         @vals = $exto || $exfrom ? () : $from;
     }
+
+    # <M..N>
     elsif ($incr>0) {
         while (1) {
-            last if  $exto && ($from >= $to || $from eq $to) 
+            last if  $exto && ($from >= $to || $from eq $to)
                  || !$exto && $from > $to;
             push @vals, $from;
             $from += $incr;
         }
     }
+
+    # <N..M>
     elsif ($incr<0) {
         while (1) {
-            last if  $exto && ($from <= $to || $from eq $to) 
+            last if  $exto && ($from <= $to || $from eq $to)
                  || !$exto && $from < $to;
             push @vals, $from;
             $from += $incr;
@@ -222,18 +278,30 @@ sub _gen_range {
         croak "Bad filter ($filter): $@" if $@;
     }
 
+    # Apply any selector before returning values...
+    if (defined $selector) {
+        @vals = $selector_sub{$selector}->($count, @vals);
+    }
+
     return @vals;
 };
 
+# Simulate a Perl 6 <<...>> construct...
 sub _qww {
     my ($content) = @_;
 
+    # Strip any filter...
+    $content =~ s{$SELECTOR \Z}{}x;
+    my ($selector, $count) = ($selector_sub{$1||'all'}, $2||1);
+
     # Break into words (or "w o r d s" or 'w o r d s') and strip quoters...
-    return map { !defined($_) ? () : $_ }
+    return $selector->( $count,
+            grep { defined($_) }
                     $content =~ m{ " ( [^\\"]* (?:\\. [^\\"]*)* ) "
                                  | ' ( [^\\']* (?:\\. [^\\']*)* ) '
                                  |   ( \S+                      )
-                                 }gxms;
+                                 }gxms
+    );
 }
 
 
@@ -247,7 +315,7 @@ List::Maker - Generate more sophisticated lists than just $a..$b
 
 =head1 VERSION
 
-This document describes List::Maker version 0.0.5
+This document describes List::Maker version 0.005
 
 
 =head1 SYNOPSIS
@@ -260,7 +328,7 @@ This document describes List::Maker version 0.0.5
 
     @list = <1,3,..10>;                   # (1,3,5,7,9)
     @list = <1..10 x 2>;                  # (1,3,5,7,9)
-  
+
     @list = <0..10 : prime N>;            # (2,3,5,7)
     @list = <1,3,..30  : /7/>;            # (7,17,27)
 
@@ -283,8 +351,8 @@ This document describes List::Maker version 0.0.5
 
 =head1 DESCRIPTION
 
-The List::Maker module hijacks Perl's built-in file globbing syntax (C<< <
-*.pl > >> and C<glob '*.pl'>) and retargets it at list creation.
+The List::Maker module hijacks Perl's built-in file globbing syntax
+(C<< < *.pl > >> and C<glob '*.pl'>) and retargets it at list creation.
 
 The rationale is simple: most people rarely if ever glob a set of files,
 but they have to create lists in almost every program they write. So the
@@ -294,15 +362,36 @@ Alternatively, you can load the module with an explicit name, and it creates a
 subroutine of that name that builds the same kinds of lists for you (leaving
 the globbing mechanism unaltered).
 
-=head1 INTERFACE 
+=head1 INTERFACE
 
 Within any file in which the module has been explicitly loaded:
 
     use List::Maker;
-    
+
 angle brackets no longer expand a shell pattern into a list of files.
 Instead, they expand a list specification into a list of values.
 
+Under Perl 5.10 and later the module is also lexically scoped in its
+effects. That is, under Perls that support it, the change in the
+behaviour of angle brackets is confined to the specific
+lexical scope into which the module was imported.
+
+Under Perl 5.8 and earlier, loading the module changes the effect
+of C<< <...> >> and C<glob()> for the remainder of the current
+package in the current file.
+
+This means:
+
+    # Code                  Under 5.8 or earlier    Under 5.10 or later
+    ====================    ====================    ===================
+    @list = <1..10>;        normal glob()           normal glob()
+    {
+        use List::Maker;    installs in file        installs in block
+        @list = <1..10>;    generates list          generates list
+    }
+    @list = <1..10>;        generates list          normal glob()
+
+
 =head2 Numeric lists
 
 Numeric list specifications may take any of the following forms:
@@ -320,7 +409,7 @@ The numbers don't have to be integers either:
     @scores = <0.5..4.5>;      # same as: (0.5, 1.5, 2.5, 3.5, 4.5)
 
     @steps = <1..0 x -0.2>;    # same as: (1, 0.8, 0.6, 0.4, 0.2, 0)
-    
+
 
 =head2 Filtered numeric lists
 
@@ -361,6 +450,41 @@ or (since the specific letter is irrelevant):
     @available = < ^$max : !allocated{T} >
 
 
+=head2 Randomly selecting from lists
+
+In addition to (or instead of) specifying a filter on a list,
+you can also select a specific number of the list's items
+at random, by appending C<:pick> I<N> to the list specification.
+
+For example:
+
+    $N_random_percentages = <0..100 : pick $N >;
+
+    @any_three_primes = <3,5..99> : is_prime(I) : pick 3>;
+
+    $one_available = < ^$max : !allocated{T} :pick>
+
+The requested number of elements are picked at random, and without
+replacement. If the number of elements to be picked is omitted,
+a single element is randomly picked.
+
+You can also pick I<with> replacement (which is equivalent to rolling
+some number of M-sided dice, with one list element on each face), by
+using C<:roll> instead of C<:pick>:
+
+    $N_nonunique_random_percentages = <0..100 : roll $N >;
+
+    @three_nonunique_primes = <3,5..99> : is_prime(I) : roll 3>;
+
+    $one_available = < ^$max : !allocated{T} :roll>
+
+If the number of elements to be rolled is omitted, a single element is
+randomly rolled (which is exactly the same as randomly picking it).
+
+Note that, because each requested "roll" is independent, it's entirely
+possible for one or more values to be selected repeatedly.
+
+
 =head2 String lists
 
 Any list specification that doesn't conform to one of the four pattern
@@ -379,13 +503,13 @@ listification):
 More interestingly, the words in these lists can be quoted to change the
 default whitespace separation. For example:
 
-    @names = <Tom Dick "Harry Potter">;   
+    @names = <Tom Dick "Harry Potter">;
                         # same as: ( 'Tom', 'Dick', 'Harry Potter' )
-    
+
 Single quotes may be also used, but this may be misleading, since the
 overall list still interpolates in that case:
 
-    @names = <Tom Dick '$Harry{Potter}'>;   
+    @names = <Tom Dick '$Harry{Potter}'>;
                         # same as: ( 'Tom', 'Dick', "$Harry{Potter}" )
 
 In a scalar context, any string list is converted to the standard
@@ -446,7 +570,7 @@ that they are defined, but I<before> the standard built-in formats
 described above.
 
 
-=head1 ALTERNATE INTERFACE 
+=head1 ALTERNATE INTERFACE
 
 If an argument is passed to the C<use List::Maker> statement, then that
 argument is used as the name of a subroutine to be installed in the current
@@ -475,7 +599,7 @@ For example:
 
     use List::Maker 'conjoin';
 
-    print conjoin @names;
+    print scalar conjoin @names;
 
 
 =head1 DIAGNOSTICS
